let Level = {};
Level.DATA = 0;
Level.INFO = 1;
Level.WARN = 2;
Level.ERROR = 3;
Level.__NOTHING = 4;
Level[Level.DATA] = "DATA";
Level[Level.INFO] = "INFO";
Level[Level.WARN] = "WARN";
Level[Level.ERROR] = "ERROR";
Level[Level.__NOTHING] = "__NOTHING";

class Display {
    /**
     * @param {?} message
     * @param {?} params
     * @param {?} moduleName
     * @param {?} moduleColor
     * @param {?} level
     * @param {?} moduleWidth
     * @return {?}
     */
    static msg(message, params, moduleName, moduleColor, level, moduleWidth) {
        if (typeof window != 'undefined' && window.document) {
            let /** @type {?} */ color = 'gray';
            if (level === Level.INFO)
                color = 'deepskyblue';
            if (level === Level.ERROR)
                color = 'red';
            if (level === Level.WARN)
                color = 'orange';
            if (moduleWidth) {
                const /** @type {?} */ diff = moduleWidth - moduleName.length;
                if (diff > 0) {
                    for (let /** @type {?} */ i = 0; i < diff; i++) {
                        moduleName += ' ';
                    }
                }
            }
            if (typeof message === 'string') {
                let /** @type {?} */ a1 = '%c ' + moduleName + '  %c ' + message + ' ';
                let /** @type {?} */ a2 = 'background: ' + moduleColor + ';color:white; border: 1px solid ' + moduleColor + '; ';
                let /** @type {?} */ a3 = 'border: 1px solid ' + color + '; ';
                params.unshift(a3);
                params.unshift(a2);
                params.unshift(a1);
            }
            else {
                let /** @type {?} */ a1 = '%c ' + moduleName + ' ';
                let /** @type {?} */ a2 = 'background: ' + moduleColor + ';color:white; border: 1px solid ' + color + '; ';
                params.push(message);
                params.unshift(a2);
                params.unshift(a1);
            }
        }
        else {
            params.unshift('[' + moduleName + '] ' + message);
        }
        console.log.apply(console, params);
    }
}

/**
 * @param {?} arr
 * @param {?} item
 * @return {?}
 */
function contain(arr, item) {
    return arr.filter(l => l === item || ((item.match && typeof item.match === 'function') ? item.match(l) : false)).length > 0;
}

class Logger {
    /**
     * @param {?} name
     * @param {?} color
     * @param {?} developmentMode
     * @param {?} allowed
     * @param {?} isMuted
     * @param {?} fixedWidth
     * @param {?=} display
     */
    constructor(name, color, developmentMode, allowed, isMuted, fixedWidth, display) {
        this.name = name;
        this.color = color;
        this.developmentMode = developmentMode;
        this.allowed = allowed;
        this.isMuted = isMuted;
        this.fixedWidth = fixedWidth;
        this.display = display;
        /**
         * @deprecated Use data(...)
         * @see data
         */
        this.d = (name, ...data) => this._data(name, data);
        /**
         * @deprecated Use error(...)
         * @see error
         */
        this.er = (name, ...data) => this._error(name, data);
        /**
         * @deprecated Use info(...)
         * @see info
         */
        this.i = (name, ...data) => this._info(name, data);
        /**
         * @deprecated Use warn(...)
         * @see warn
         */
        this.w = (name, ...data) => this._warn(name, data);
        /**
         * Logs message and data with the level=data
         * @param message The message
         * @param otherParams Additional parameters
         */
        this.data = (message, ...otherParams) => { return this._data(message, otherParams); };
        /**
         * Logs message and data with the level=error
         * @param message The message
         * @param otherParams Additional parameters
         */
        this.error = (message, ...otherParams) => this._error(message, otherParams);
        /**
         * Logs message and data with the level=info
         * @param message The message
         * @param otherParams Additional parameters
         */
        this.info = (message, ...otherParams) => this._info(message, otherParams);
        /**
         * Logs message and data with the level=warn
         * @param message The message
         * @param otherParams Additional parameters
         */
        this.warn = (message, ...otherParams) => this._warn(message, otherParams);
    }
    /**
     * @param {?} name
     * @param {...?} data
     * @return {?}
     */
    _data(name, ...data) {
        if (this.allowed.length >= 1 && contain(this.allowed, Level.__NOTHING)
            && !contain(this.allowed, Level.DATA))
            return this;
        if (Logger.isProductionMode)
            return this;
        if (this.display !== undefined)
            this.display(name, data, Level.DATA, this.name);
        else if (this.allowed.length === 0 || contain(this.allowed, Level.DATA)) {
            Display.msg.apply(undefined, [name, ...data, this.name, this.color, Level.DATA, this.fixedWidth]);
        }
        return this;
    }
    /**
     * @param {?} name
     * @param {...?} data
     * @return {?}
     */
    _error(name, ...data) {
        if (this.allowed.length >= 1 && contain(this.allowed, Level.__NOTHING)
            && !contain(this.allowed, Level.ERROR))
            return this;
        if (Logger.isProductionMode)
            return this;
        if (this.display !== undefined)
            this.display(name, data, Level.ERROR, this.name);
        else if (this.allowed.length === 0 || contain(this.allowed, Level.ERROR)) {
            Display.msg.apply(undefined, [name, ...data, this.name, this.color, Level.ERROR, this.fixedWidth]);
        }
        return this;
    }
    /**
     * @param {?} name
     * @param {...?} data
     * @return {?}
     */
    _info(name, ...data) {
        if (this.allowed.length >= 1 && contain(this.allowed, Level.__NOTHING)
            && !contain(this.allowed, Level.INFO))
            return this;
        if (Logger.isProductionMode)
            return this;
        if (this.display !== undefined)
            this.display(name, data, Level.INFO, this.name);
        else if (this.allowed.length === 0 || contain(this.allowed, Level.INFO)) {
            Display.msg.apply(undefined, [name, ...data, this.name, this.color, Level.INFO, this.fixedWidth]);
        }
        return this;
    }
    /**
     * @param {?} name
     * @param {...?} data
     * @return {?}
     */
    _warn(name, ...data) {
        if (this.allowed.length >= 1 && contain(this.allowed, Level.__NOTHING)
            && !contain(this.allowed, Level.WARN))
            return this;
        if (Logger.isProductionMode)
            return this;
        if (this.display !== undefined)
            this.display(name, data, Level.WARN, this.name);
        else if (this.allowed.length === 0 || contain(this.allowed, Level.WARN)) {
            Display.msg.apply(undefined, [name, ...data, this.name, this.color, Level.WARN, this.fixedWidth]);
        }
        return this;
    }
    /**
     * @param {?} name
     * @param {?} level
     * @param {...?} data
     * @return {?}
     */
    _logMessage(name, level, ...data) {
        if (this.isMuted)
            return this;
        if (this.allowed.length >= 1 && contain(this.allowed, level)
            && !contain(this.allowed, level))
            return this;
        if (Logger.isProductionMode)
            return this;
        if (this.display !== undefined)
            this.display(name, data, level, this.name);
        else if (this.allowed.length === 0 || contain(this.allowed, level)) {
            Display.msg(name, data, this.name, this.color, level, this.fixedWidth);
        }
        return this;
    }
    /**
     * @param {?} l
     * @return {?}
     */
    level(l) {
        this._level = l;
        return this;
    }
    /**
     * @return {?}
     */
    mute() {
        this.isMuted = true;
    }
}
Logger.isProductionMode = false;

class Log {
    /**
     * @template TA
     * @param {?} name
     * @param {...?} level
     * @return {?}
     */
    static create(name, ...level) {
        let /** @type {?} */ i;
        if (Log.instances[name] === undefined) {
            i = new Logger(name, Log.getRandomColor(), Log.isDevelopmentMode, level, Log.isMutedModule(name), Log.levels.length > 0 ? Log.fixedWidth : undefined, Log.levels.length > 0 ? Log.display : undefined);
            Log.instances[name] = i;
        }
        else {
            i = Log.instances[name];
        }
        return i;
    }
    /**
     * @return {?}
     */
    static getRandomColor() {
        let /** @type {?} */ letters = '012345'.split('');
        let /** @type {?} */ color = '#';
        color += letters[Math.round(Math.random() * 5)];
        letters = '0123456789ABCDEF'.split('');
        for (let /** @type {?} */ i = 0; i < 5; i++) {
            color += letters[Math.round(Math.random() * 15)];
        }
        if (color === undefined)
            return this.getRandomColor();
        return color;
    }
    /**
     * @param {?} name
     * @param {?} data
     * @param {?} incomming
     * @param {?} moduleName
     * @return {?}
     */
    static display(name, data, incomming, moduleName) {
        if (!contain(Log.levels, incomming))
            return;
        if (incomming === Level.DATA) {
            Display.msg(name, data, name, Log.instances[moduleName].color, Level.DATA, Log.instances[moduleName].fixedWidth);
        }
        if (incomming === Level.ERROR) {
            Display.msg(name, data, name, Log.instances[moduleName].color, Level.ERROR, Log.instances[moduleName].fixedWidth);
        }
        if (incomming === Level.INFO) {
            Display.msg(name, data, name, Log.instances[moduleName].color, Level.INFO, Log.instances[moduleName].fixedWidth);
        }
        if (incomming === Level.WARN) {
            Display.msg(name, data, name, Log.instances[moduleName].color, Level.WARN, Log.instances[moduleName].fixedWidth);
        }
    }
    /**
     * @param {...?} level
     * @return {?}
     */
    static onlyLevel(...level) {
        if (Log._logOnly) {
            console.error('You should use funcion onlyLevel only once');
            return;
        }
        if (Log._logOnly)
            Log._logOnly = true;
        if (level.length === 0)
            return;
        Log.levels = level;
    }
    /**
     * @param {...?} modules
     * @return {?}
     */
    static onlyModules(...modules) {
        if (Log._logModules) {
            console.error('You should use funcion onlyModules only once');
            return;
        }
        if (modules.length === 0)
            return;
        Log.modules = modules;
        Log.muteAllOtherModules();
    }
    /**
     * @param {?} moduleName
     * @return {?}
     */
    static isMutedModule(moduleName) {
        if (Log.modules.length == 0)
            return false;
        if (!contain(Log.modules, moduleName))
            return true;
        return false;
    }
    /**
     * @return {?}
     */
    static muteAllOtherModules() {
        for (var /** @type {?} */ moduleName in Log.instances) {
            if (!contain(Log.modules, moduleName))
                Log.instances[moduleName].mute();
        }
    }
    /**
     * @return {?}
     */
    static setProductionMode() {
        if (Log.modeIsSet) {
            console.error('Mode is already set');
            return;
        }
        if (console !== undefined && console.clear !== undefined) {
            setTimeout(() => {
                console.clear();
                console.log = () => { };
                console.error = () => { };
                console.warn = () => { };
                console.info = () => { };
            });
        }
        Logger.isProductionMode = true;
        Log.isDevelopmentMode = false;
    }
}
Log.instances = {};
Log.fixedWidth = 0;
Log._logOnly = false;
Log.levels = [];
Log._logModules = false;
Log.modules = [];
Log.isDevelopmentMode = true;
Log.modeIsSet = false;

/**
 * Generated bundle index. Do not edit.
 */

export { Log, Level };
//# sourceMappingURL=ng2-logger.js.map
